package com.ruoyi.ss.refund.service.impl;

import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.refund.mapper.RefundMapper;
import com.ruoyi.ss.refund.service.IRefundService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

import static com.ruoyi.common.constant.ServiceConstants.REFUND_STATUS_SUCCESS;

/**
 * 退款订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@Service
public class RefundServiceImpl implements IRefundService
{
    @Resource
    private RefundMapper refundMapper;

    @Resource
    private IOrderService orderService;

    /**
     * 查询退款订单
     *
     * @param id 退款订单主键
     * @return 退款订单
     */
    @Override
    public RefundVO selectEtRefundById(Long id)
    {
        return refundMapper.selectEtRefundById(id);
    }

    /**
     * 根据订单号查询退款订单对象
     *
     * @param orderNo 订单号
     * @return 退款订单
     */
    @Override
    public List<Refund> selectEtRefundByOrderNo(String orderNo)
    {
        return refundMapper.selectEtRefundByOrderNo(orderNo);
    }

    /**
     * 根据退款单号查询退款订单对象
     *
     * @param refundNo 退款订号
     * @return 退款订单
     */
    @Override
    public Refund selectEtRefundByRefundNo(String refundNo)
    {
        return refundMapper.selectEtRefundByRefundNo(refundNo);
    }


    /**
     * 查询退款订单列表
     *
     * @param rlRefund 退款订单
     * @return 退款订单
     */
    @Override
    public List<Refund> selectEtRefundList(Refund rlRefund)
    {
        return refundMapper.selectEtRefundList(rlRefund);
    }

    /**
     * 查询退款订单列表
     *
     * @param rlRefund 退款订单
     * @return 退款订单
     */
    @Override
    @DataScope(merchantAlias = "m", userAlias = "u")
    public List<Refund> selectEtRefundListWithIsolate(Refund rlRefund)
    {
        return refundMapper.selectEtRefundList(rlRefund);
    }

    /**
     * 新增退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    @Override
    public int insertEtRefund(Refund rlRefund)
    {
        OrderVO orderVO = orderService.selectRlOrderByOrderNo(rlRefund.getOrderNo());
        ServiceUtil.assertion(orderVO == null, "订单不存在");
        rlRefund.setCreateTime(DateUtils.getNowDate());
        rlRefund.setUserId(SecurityUtils.getUserId());
        rlRefund.setUserName(SecurityUtils.getUsername());
        rlRefund.setRefundNo(IdUtils.getOrderNo(ServiceConstants.ORDER_TYPE_REFUND_PREFIX));
        rlRefund.setMerchantId(orderVO.getMerchantId());
        return refundMapper.insertEtRefund(rlRefund);
    }

    /**
     * 修改退款订单
     *
     * @param rlRefund 退款订单
     * @return 结果
     */
    @Override
    public int updateEtRefund(Refund rlRefund)
    {
        return refundMapper.updateEtRefund(rlRefund);
    }

    /**
     * 批量删除退款订单
     *
     * @param ids 需要删除的退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundByIds(Long[] ids)
    {
        return refundMapper.deleteEtRefundByIds(ids);
    }

    /**
     * 删除退款订单信息
     *
     * @param id 退款订单主键
     * @return 结果
     */
    @Override
    public int deleteEtRefundById(Long id)
    {
        return refundMapper.deleteEtRefundById(id);
    }

    /**
     * 根据退款单号修改退款订单
     * @param rlRefund
     * @return
     */
    @Override
    public int updateEtRefundByRefundNo(Refund rlRefund) {
        return refundMapper.updateEtRefundByRefundNo(rlRefund);
    }

    /**
     * 判断是否用户是否还有未退款的押金
     *
     * @param userId 用户id
     * @return 结果
     */
    @Override
    public List<Order> checkUserDeposit(Long userId) {
       return refundMapper.checkUserDeposit(userId);
    }

    /**
     * 统计已退款金额
     *
     * @param orderNo 订单号
     * @return 结果
     */
    @Override
    public RefundVO getRefundedAmount(String orderNo) {
        return refundMapper.getRefundedAmount(orderNo);
    }

    /**
     * 同意退款
     */
    @Override
    public int passRefund(String refundNo) {
        Refund refund = refundMapper.selectEtRefundByRefundNo(refundNo);
        ServiceUtil.assertion(refund == null, "退款申请单不存在");
        ServiceUtil.assertion(!ServiceConstants.REFUND_STATUS_PENDING.equals(refund.getStatus()), "退款申请单已处理,请勿操作！");

        // TODO 根据支付渠道 调用退款方法  2. 更新退款单状态为 REFUND_STATUS_SUCCESS
        Refund refund1 = new Refund();
        refund1.setRefundNo(refundNo);
        refund1.setStatus(REFUND_STATUS_SUCCESS);
        int i = refundMapper.updateEtRefundByRefundNo(refund);
        ServiceUtil.assertion(i == 0, "退款申请单更新失败");

        return 1;
    }

    /**
     * 拒绝退款
     */
    @Override
    public int reject(String refundNo) {
        Refund refund = refundMapper.selectEtRefundByRefundNo(refundNo);
        ServiceUtil.assertion(refund == null, "退款申请单不存在");
        ServiceUtil.assertion(!ServiceConstants.REFUND_STATUS_PENDING.equals(refund.getStatus()), "退款申请单已处理,请勿操作！");
        Refund update  = new Refund();
        update.setRefundNo(refundNo);
        update.setStatus(ServiceConstants.REFUND_STATUS_REJECTED);
        int i = refundMapper.updateEtRefundByRefundNo(update);
        ServiceUtil.assertion(i == 0, "退款申请单更新失败");
        return i;
    }

}
